import os
from django.http import Http404
from django.utils.translation import to_locale, get_language
from django.contrib.gis.geoip2 import GeoIP2
from django.conf import settings
from .models import Copy


GEOIP_PATH = settings.GEOIP_PATH


def get_ip_address(request):
    "Map request to external IP address resolving internal address if necessary"

    standardised_headers = {key.lower(): value for key, value in request.headers.items()}

    if "x-forwarded-for" in standardised_headers:
        return standardised_headers.get("x-forwarded-for").split(',')[0]
    elif "forwarded" in standardised_headers:
        # Header format: "Forwarded": "for=<for_ip>, for=<proxy_ip>;host=<host>;proto=https"
        # Extract the for=... component
        forwarded_for = next(
            component.lower() for component in request.headers.get("Forwarded").split(";")
            if component.lower().startswith("for")
        )
        # Get the first IP address in the redirect chain (original client)
        ip_address = forwarded_for.strip("for=").split(",")[0]

        return ip_address
    else:
        return request.META.get('REMOTE_ADDR')


def ip_to_country_code(addr, default_code='GB'):
    "Map request to client's country based on IP address"

    if addr == '127.0.0.1' or addr == 'localhost':
        return default_code

    g = GeoIP2(path=GEOIP_PATH)
    return g.country(addr)['country_code']


def get_client_country_code(request):
    "Shorthand to request country code directly from request"
    return ip_to_country_code(get_ip_address(request))


class LocalisationMiddleware:
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        url = request.path
        locale = to_locale(get_language())
        geo = get_client_country_code(request)
        draft = request.user.is_authenticated and 'draft' in request.GET

        request.copy = Copy.get_for_url(url, locale, geo, draft)

        return self.get_response(request)



# Generated by Django 2.1.1 on 2018-10-13 14:53

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields
import simple_history.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('djangocopy', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='HistoricalCopy',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('url', models.CharField(blank=True, help_text='URL name (leave empty to load for all templates)', max_length=255)),
                ('fieldid', models.SlugField(help_text='The field identifier that will be used in templates', max_length=100)),
                ('locale', models.CharField(blank=True, help_text="Browser settings (e.g. 'en_GB')", max_length=5)),
                ('geo', models.CharField(blank=True, help_text="Country code derived from the IP (e.g. 'GB')", max_length=2)),
                ('text', models.TextField(max_length=10000)),
                ('format', models.CharField(choices=[('p', 'Plain text'), ('m', 'Markdown'), ('j', 'JSON'), ('h', 'HTML'), ('s', 'Special HTML')], default='p', max_length=1)),
                ('status', models.CharField(choices=[('d', 'Draft'), ('p', 'Published')], default='d', max_length=1)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_date', models.DateTimeField()),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical copy',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='HistoricalNavbar',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('logo', models.TextField(blank=True, help_text='A picture to use as a logo', max_length=100, null=True)),
                ('elements', jsonfield.fields.JSONField()),
                ('z_index', models.IntegerField(default=0, help_text='The z-index determines the order of navbar items. A higher value appears first.')),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_date', models.DateTimeField()),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'historical navbar',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name='HistoricalPage',
            fields=[
                ('id', models.IntegerField(auto_created=True, blank=True, db_index=True, verbose_name='ID')),
                ('slug', models.SlugField(max_length=255)),
                ('authenticated', models.BooleanField(default=False, help_text='If True, visitor must login to access this page')),
                ('title', models.CharField(blank=True, default='', max_length=255)),
                ('description', models.CharField(blank=True, default='', max_length=255)),
                ('keywords', models.CharField(blank=True, default='', max_length=255)),
                ('history_id', models.AutoField(primary_key=True, serialize=False)),
                ('history_change_reason', models.CharField(max_length=100, null=True)),
                ('history_date', models.DateTimeField()),
                ('history_type', models.CharField(choices=[('+', 'Created'), ('~', 'Changed'), ('-', 'Deleted')], max_length=1)),
                ('history_user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('template', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='djangocopy.Template')),
            ],
            options={
                'verbose_name': 'historical page',
                'ordering': ('-history_date', '-history_id'),
                'get_latest_by': 'history_date',
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
    ]



from django.template import Library
from djangocopy.models import Page, Copy, Navbar


register = Library()


@register.inclusion_tag('djangocopy/navbar.html', takes_context=True)
def navbar(context):
    """
    Generate navbar

    DEVNOTE: Consider using Django-Sites to have difference navbar of each site
    """

    if hasattr(context, 'request') and hasattr(context.request, 'user'):
        # First check if there are Navbar associated with this user's group, then look for a default one
        qs = Navbar.objects.filter(groups__in=context.request.user.groups.all()).order_by('-z_index').distinct()
        if not qs.exists():
            qs = Navbar.objects.filter(groups=None) # return default navbar
        if not qs.exists():
            return {} # still nothing? return empty

    else:
        return {}

    logo = None
    elements = []
    for nav in qs:
        if nav.logo and not logo: logo = nav.logo.image.url
        elements += nav.elements

    #for e in navbar.elements:
    #    print(e)
    #    e['url'] = reverse(e['url'], args=e.get('args',None))

    return {
        'navbar_logo': logo, # if there are several navbars use only the first logo
        'navbar_items': elements,
    }


from html2text import html2text as h2t


def choices_as_string(choices, param, default="--"):
    return dict(choices).get(param, default)



def html2text(text):
    return h2t(text).replace('#','').replace('**','').replace('__','')

    


from uuid import uuid4
from django.utils.safestring import mark_safe
from django.utils.translation import to_locale, get_language
from django.template import Library, Node, TemplateSyntaxError

from ..models import Copy


register = Library()


@register.filter()
def numeric_range(size):
    "Convert integer to a range (for numeric for loops generally)"
    return range(size)


@register.simple_tag
def getcopy(id):
    lang = to_locale(get_language())
    lang = 'en_GB' if lang == 'en_GB' or lang == 'en' else 'en'
    copy = Copy.objects.values_list('fieldid','text_'+lang)
    return dict(copy)


@register.simple_tag
def glyphicon(name,aria_text=None,options=None,aria_hidden=False):
    "HTML for inserting a Bootstrap 3 glyph icon. 'aria_hidden' should be True if icon is purely decorative."
    return mark_safe('<span class="glyphicon glyphicon-{} {}" aria-hidden="{}" aria-text="{}"></span>'.format(name,options,str(aria_hidden).lower(),aria_text))


@register.simple_tag
def faicon(name, aria_text='', options='', aria_hidden=True):
    "HTML for inserting a Font Awesome icon. 'aria_hidden' should be True if icon is purely decorative. 'options' can be fa-lg, fa-order, fa-pull-right and others"
    return mark_safe('<i class="fa fa-{} {}" aria-hidden="{}" aria-text="{}"></i>'.format(name,options,str(aria_hidden).lower(),aria_text))


@register.filter
def list_to_2_column(objects):
    "Convert a list into 2 sub-lists of similar size. Usually used to break a long list of objects into columns"
    sz = int(len(objects) / 2. + 0.51)
    if sz:
        return [objects[:sz],objects[sz:]]
    else:
        return [objects]


@register.filter
def list_to_3_column(objects):
    "Convert a list into 3 sub-lists of similar size. Usually used to break a long list of objects into columns"
    sz = int(len(objects) / 3. + 0.63)
    if sz:
        return [objects[:sz],objects[sz:2*sz],objects[2*sz:]]
    else:
        return [objects]


@register.filter
def list_to_4_column(objects):
    "Convert a list into 4 sub-lists of similar size. Usually used to break a long list of objects into columns"
    sz = int(len(objects) / 4. + 0.76)
    if sz:
        return [objects[:sz],objects[sz:2*sz],objects[2*sz:3*sz],objects[3*sz:]]
    else:
        return [objects]


# Based on UUID Template Tag, https://djangosnippets.org/snippets/1356/
#

class UUIDNode(Node):
    def __init__(self, var_name):
        self.var_name = var_name

    def render( self, context ):
        context[self.var_name] = str(uuid4())
        return ''

@register.tag(name='uuid')
def uuid_tag(parser,token):
    """
    Generate a random UUID and store it in a named context variable.

    Sample usage:
        {% uuid varname %}
        varname will contain the generated UUID
    """
    try:
        tagname, varname = token.contents.split()
    except ValueError:
        raise TemplateSyntaxError("{} tag requires exactly one argument".format(token.contents.split()[0]))

    return UUIDNode(varname)
